(define-module (wip android)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix build-system android-ndk)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (gnu packages)
  #:use-module (gnu packages android))

(define (android-platform-build version)
  (origin
    (method git-fetch)
    (uri (git-reference
          (url "https://android.googlesource.com/platform/build")
          (commit (string-append "android-" version))))
    (file-name (string-append "android-platform-build-"
                              version "-checkout"))
    (sha256
     (base32
      "0a6621i9jjv9w0p2m6z5y5ljzxmjalrmi082vahhf00wx3ifv0ld"))))

(define-public android-build
  (package
    (name "android-build")
    (version "0.5.5-2")
    (source (android-platform-build (android-platform-version)))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (delete 'configure)
         (delete 'build)
         (delete 'check)
         (replace 'install
             (lambda* (#:key outputs #:allow-other-keys)
               (let* ((out (assoc-ref outputs "out"))
                      (share (string-append out "/share/android/build")))
                 (mkdir-p share)
                 (copy-recursively "." share)
                 #t))))))
    (home-page "https://android.googlesource.com/platform/build/+/master/core/build-system.html")
    (synopsis "Build system of the Android NDK platform")
    (description "@code{android-build} provides the Android NDK
build system.")
    (license license:asl2.0)))

(define-public android-fm-si4709
  (let ((revision "1")
        (commit "e106292d05917889f20495d99fb0cf0930639f61"))
    (package
      (name "android-fm-si4709")
      (version (git-version "0.1" revision commit))
      (source (origin
        (method git-fetch)
    (uri (git-reference
          (url "https://github.com/PinZhang/fm-Si4709.git")
          (commit commit)))
    (file-name (string-append "android-fm-si4709-"
                              version "-checkout"))
    (sha256
     (base32
      "0l7nl6bplcy8vr0gr6g1ckdr8l8g8kfs5mjzg8fq89jkyfdsg3bb"))))
    (build-system android-ndk-build-system)
    (home-page "https://developer.android.com/")
    (synopsis "Android FM radio frontend")
    (description "This package provides @command{pushfm.sh}, a command to
tune the Si4709 FM radio.")
    (license license:asl2.0))))

(define (android-platform-dalvik version)
  (origin
    (method git-fetch)
    (uri (git-reference
          (url "https://android.googlesource.com/platform/dalvik")
          (commit (string-append "android-" version))))
    (file-name (string-append "android-platform-dalvik-"
                              version "-checkout"))
    (sha256
     (base32
      "0m4mc7kg3pw73xjmhsygvd9551d9p7svw7p6vcjs2hczqm1z303d"))))

(define-public android-dalvik
  (package
    (name "android-dalvik")
    (version (android-platform-version))
    (source (android-platform-dalvik version))
    (build-system android-ndk-build-system)
    (home-page "https://developer.android.com/")
    (synopsis "Android Java VM")
    (description "@code{dalvik} is a Java Virtual Machine.")
    (license license:asl2.0)))
