(define-module (wip dotnet-core)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix utils)
  #:use-module (guix build-system ant)
  #:use-module (guix build-system gnu)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  #:use-module (gnu packages java)
  #:use-module (gnu packages maven)
  #:use-module (gnu packages mono)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages python)
  #:use-module (gnu packages qt))

;; TODO: Older, simpler, version: 2.1.403
(define-public dotnet-cli-stage0
  (package
    (name "dotnet-cli-stage0")
    (version "2.1.403")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://github.com/dotnet/cli/archive/v" version "+dependencies.tar.gz"))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "13zx8k5yla0ghk3j2wjrjhvabwhhnr8rdbaga4f5g0zxgfk7f4qq"))))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'delete-bundles
           (lambda _
             (for-each delete-file (find-files "." "\\.exe$"))
             (for-each delete-file (find-files "." "\\.dll$"))
             #t))
         (delete 'configure)
         (replace 'build
           (lambda* (#:key inputs #:allow-other-keys)
; build.sh --architecture x64 --skip-prereqs --nopackage --stage0 ...
             (apply invoke "mcs" "-langversion:experimental" (find-files "src/dotnet" "\\.cs$"))))
         (replace 'check
           (const #t)))))
    (native-inputs
     `(("mono" ,mono)))
    (synopsis "@command{dotnet} executable")
    (description "FIXME")
    (home-page "https://github.com/dotnet")
    (license license:asl2.0)))

(define-public dotnet-cli
  (package
    (name "dotnet-cli")
    (version "2.2.105")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://github.com/dotnet/cli/archive/v" version "+dependencies.tar.gz"))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "1yy0nlb8vz65yqln7h44gs334rh6dfl3w05biads5xcj5pzv0bzx"))))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'delete-bundles
           (lambda _
             (for-each delete-file (find-files "." "\\.exe$"))
             (for-each delete-file (find-files "." "\\.dll$"))
             #t))
         (delete 'configure)
         (replace 'build
           (lambda* (#:key inputs #:allow-other-keys)
; TODO: --architecture x64 --skip-prereqs --nopackage
             (invoke "./build.sh" "--configuration" "Release" "--stage0"
                     (assoc-ref inputs "stage0"))))
         (replace 'check
           (lambda _
             (invoke "./build.sh" "--test"))))))
    (native-inputs
     `(("stage0" ,dotnet-cli-stage0)))
    (inputs
     `())
    (synopsis "@command{dotnet} executable")
    (description "FIXME")
    (home-page "https://github.com/dotnet")
    (license license:asl2.0)))

(define-public roslyn
  (package
    (name "roslyn")
    (version "2.10.0")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://github.com/dotnet/roslyn/archive/version-" version ".tar.gz"))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "1sar2zgrlx59zzrijb5bq7wzyzz3s9gnawpjb8qpxc13yxa3945v"))))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'delete-bundles
           (lambda _
             (for-each delete-file (find-files "." "\\.exe$"))
             (for-each delete-file (find-files "." "\\.dll$"))
             #t))
         (delete 'configure)
         (replace 'build
           (lambda _
             (invoke "./build.sh" "--release")))
         (replace 'check
           (lambda _
             (invoke "./build.sh" "--test"))))))
    (native-inputs
     `())
    (inputs
     `())
    (synopsis "Roslyn .NET compiler (C#, Visual Basic)")
    (description "FIXME")
    (home-page "https://github.com/dotnet")
    (license license:asl2.0)))

(define-public corefx
  (package
    (name "corefx")
    (version "2.2.3")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://github.com/dotnet/corefx/archive/v" version ".tar.gz"))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "0faqk4hmrxd6866pm9y7mi23gz5msqv8s9pq1z2a6vs3f4hjcbz3"))))
    (build-system gnu-build-system)
    (native-inputs
     `())
    (inputs
     `())
    (synopsis "Foundational class libraries for .NET")
    (description "FIXME")
    (home-page "https://github.com/dotnet")
    (license license:expat)))

(define-public coreclr
  (package
    (name "coreclr")
    (version "2.2.3")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://github.com/dotnet/coreclr/archive/v" version ".tar.gz"))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "1kq5f6ax85dglb0gs8nlrbq7clhc1ih18l0vxpbasbs41yrcfhm9"))))
    (build-system gnu-build-system)
    (native-inputs
     `())
    (inputs
     `())
    (synopsis "Common language runtime for .NET")
    (description "FIXME")
    (home-page "https://github.com/dotnet")
    (license license:expat)))
