
(define-module (wip dwave)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix hg-download)
  #:use-module (guix utils)
  #:use-module (guix build-system python)
  #:use-module (guix build-system gnu)
  #:use-module (guix build-system cmake)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  #:use-module (gnu packages base)
  #:use-module (gnu packages boost)
  #:use-module (gnu packages check)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages databases)
  #:use-module (gnu packages libffi)
  #:use-module (gnu packages libusb)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages logging)
  #:use-module (gnu packages ncurses)
  #:use-module (gnu packages libftdi)
  #:use-module (gnu packages tcl)
  #:use-module (gnu packages tls)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages time)
  #:use-module (gnu packages popt)
  #:use-module (gnu packages python)
  #:use-module (gnu packages python-xyz)
  #:use-module (gnu packages statistics)
  #:use-module (gnu packages python-crypto)
  #:use-module (gnu packages python-web)
  #:use-module (gnu packages swig)
  #:use-module (gnu packages xml))

(define-public python-dwave-networkx
  (package
    (name "python-dwave-networkx")
    (version "0.6.6")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwave_networkx" version))
        (sha256
          (base32
            "08zx4s9pr4yfysmwag9cxnnwdmcpzi0jh9kpzpali5r0jbdd137c"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f)) ; FIXME: Re-enable.
    (propagated-inputs
     `(("python-decorator" ,python-decorator)
       ("python-dimod" ,python-dimod)
       ("python-networkx" ,python-networkx)))
    (home-page "https://github.com/dwavesystems/dwave_networkx")
    (synopsis "NetworkX extension providing graphs and algorithms relevent to working with the D-Wave System")
  (description "This package provides a NetworkX extension for graphs and algorithms relevent to working with the D-Wave System.")
  (license license:asl2.0)))

(define-public python-dimod
  (package
    (name "python-dimod")
    (version "0.7.7")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dimod" version))
        (sha256
          (base32
            "0afw0jnvmdxfs9wj3d3k19h3wxbv6d8wr5qmsxknwmpn1qnjmbmb"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f)) ; FIXME
    (inputs
     `(("boost" ,boost)))
    (native-inputs
     `( ;("python-nose" ,python-nose)
     ))
    (propagated-inputs
     `( ;("python-enum34" ,python-enum34)
       ;("python-futures" ,python-futures) ; python2 only.
       ("python-jsonschema" ,python-jsonschema)
       ("python-networkx" ,python-networkx)
       ("python-numpy" ,python-numpy)
       ("python-pandas" ,python-pandas)
       ("python-pymongo" ,python-pymongo)
       ("python-six" ,python-six)))
    (home-page "https://github.com/dwavesystems/dimod")
    (synopsis "API for binary quadratic model samplers.")
    (description "This package provides an API for binary quadratic model
samplers.")
    (license license:asl2.0)))

(define-public python-homebase
  (package
    (name "python-homebase")
    (version "1.0.1")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "homebase" version))
        (sha256
          (base32
            "0pk35hih8kj51b6inmwdpp22j8l2i3s6spl15n2j1d4q8bghiq4y"))))
    (build-system python-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
        (add-before 'check 'setenv
          (lambda _
            (setenv "HOME" "/tmp")
            #t)))))
    (native-inputs
     `(("python-virtualenv" ,python-virtualenv)))
    (home-page "https://github.com/dwavesystems/homebase")
    (synopsis "Find paths in which applications can write data")
    (description "This package provides a platform independent library
for finding paths in which applications can write data.")
    (license license:asl2.0)))

(define-public python-dwave-neal
  (package
    (name "python-dwave-neal")
    (version "0.4.2")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwave-neal" version))
        (sha256
          (base32
            "0qzxl28cpxyyml6cpmj0p58plwivdv025rp398jj3jypmcq32gb6"))))
    (build-system python-build-system)
    (propagated-inputs
     `(("python-dimod" ,python-dimod)
       ("python-numpy" ,python-numpy)
       ("python-six" ,python-six)))
    (home-page "https://github.com/dwavesystems/dwave-neal")
    (synopsis "General Ising graph simulated annealing solver")
    (description "This package provides a general Ising graph simulated
annealing solver.")
    (license license:asl2.0)))

(define-public python-dwave-qbsolv
  (package
    (name "python-dwave-qbsolv")
    (version "0.2.9")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwave-qbsolv" version))
        (sha256
          (base32
            "0srxapxcgnbw7ayykq5gnyrcijnai4m3xp5jqfgawgbqp451b90h"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-dimod" ,python-dimod)))
    (home-page "https://github.com/dwavesystems/qbsolv")
    (synopsis "Large quadratic unconstrained binary optimization problem solver")
    (description "This provides a decomposing solver which finds a minimum
value of a large quadratic unconstrained binary optimization (QUBO) problem
by splitting it into pieces solved either via a D-Wave system or a classical
tabu solver.  @code{qbsolv} by default uses its internal classical solver.
Access to a D-Wave system must be arranged separately.")
    (license license:asl2.0)))

(define-public python-dateutil-2.7
  (package
    (inherit python-dateutil)
    (name "python-dateutil")
    (version "2.7.3")
    (source
     (origin
       (method url-fetch)
       (uri (pypi-uri "python-dateutil" version))
       (sha256
        (base32
         "1f7h54lg0w2ckch7592xpjkh8dg87k2br256h0iw49zn6bg02w72"))))
    (native-inputs
     `(("python-setuptools-scm" ,python-setuptools-scm)))))

(define-public python-requests-2.19
  (package
    (inherit python-requests)
    (name "python-requests")
    (version "2.19.1")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "requests" version))
        (sha256
          (base32
            "0snf8xxdzsgh1x2zv3vilvbrv9jbpmnfagzzb1rjmmvflckdh8pc"))))))

(define-public python-dwave-cloud-client
  (package
    (name "python-dwave-cloud-client")
    (version "0.4.15")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwave-cloud-client" version))
        (sha256
          (base32
            "1zf67dgmkfvzmm4jy6f6y2jrm5wsifr8qxbchdsci1hnxwzfjzfb"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f)) ; Requires network.
    (propagated-inputs
     `(("python-six" ,python-six)
       ("python-homebase" ,python-homebase)
       ("python-click" ,python-click)
       ("python-dateutil" ,python-dateutil-2.7)
       ("python-requests" ,python-requests-2.19)))
    (home-page "https://github.com/dwavesystems/dwave-cloud-client")
    (synopsis "Client for interacting with D-Wave cloud resources")
    (description "This package provides a client for interacting with D-Wave
cloud resources.")
    (license license:asl2.0)))

(define-public python-minorminer
  (package
    (name "python-minorminer")
    (version "0.1.5")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "minorminer" version))
        (sha256
          (base32
            "1xh61dfqnn90pv0vic37jpbl6ij1268n946xy7m181frns7cd9y8"))))
    (build-system python-build-system)
    (home-page "https://github.com/dwavesystems/minorminer")
    (synopsis "Heuristic algorithm to find graph minor embeddings")
    (description "This package provides a heuristic algorithm to find graph
minor embeddings.")
    (license license:asl2.0)))

;; TODO: Extras: dwave-drivers
(define-public python-dwave-system
  (package
    (name "python-dwave-system")
    (version "0.5.3")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwave-system" version))
        (sha256
          (base32
            "12v0dwvh7hhayay266llkri1dp5a6650ssprk3jbbc7wyxxzim69"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f)) ; Requires network
    (propagated-inputs
     `(("python-dimod" ,python-dimod)
       ("python-dwave-cloud-client" ,python-dwave-cloud-client)
       ("python-dwave-networkx" ,python-dwave-networkx)
       ("python-homebase" ,python-homebase)
       ("python-minorminer" ,python-minorminer)
       ("python-six" ,python-six)
       ("python-numpy" ,python-numpy)))
    (home-page "https://github.com/dwavesystems/dwave-system")
    (synopsis "Sampler for D-Wave System")
    (description "This package provides a sampler for the actual
physical D-Wave system.")
    (license license:asl2.0)))

(define-public python-penaltymodel
(package
  (name "python-penaltymodel")
  (version "0.15.3")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "penaltymodel" version))
      (sha256
        (base32
          "09igk3b9gb0nv9vak94v1biwfkzvc7i2igimnw331r87nc5sg062"))))
  (build-system python-build-system)
  (arguments
   `(#:tests? #f)) ; FIXME: Re-enable tests.
  (propagated-inputs
    `(("python-dimod" ,python-dimod)
      ("python-networkx" ,python-networkx)
      ;("python-penaltymodel-cache"
      ; ,python-penaltymodel-cache)
      ;("python-penaltymodel-maxgap"
      ; ,python-penaltymodel-maxgap)
      ;("python-penaltymodel-mip"
      ; ,python-penaltymodel-mip)
      ("python-six" ,python-six)))
  (home-page "https://github.com/dwavesystems/penaltymodel")
  (synopsis
    "Utilities and interfaces for using penalty models.")
  (description
    "Utilities and interfaces for using penalty models.")
  (license license:asl2.0)))

(define-public python-penaltymodel-cache
  (package
    (name "python-penaltymodel-cache")
    (version "0.3.1")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "penaltymodel-cache" version))
        (sha256
          (base32
            "12x4ml1pazibisx3q9xsfala7fr22ix1ir3shb0gqwwlp2zns781"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-penaltymodel" ,python-penaltymodel)))
    (propagated-inputs
     `(("python-dimod" ,python-dimod)
       ("python-homebase" ,python-homebase)))
    (home-page "https://github.com/dwavesystems/penaltymodel")
    (synopsis "Local cache for penalty models.")
    (description "A local cache for penalty models.")
    (license license:asl2.0)))

(define-public python-pysmt
(package
  (name "python-pysmt")
  (version "0.7.5")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "pysmt" version))
      (sha256
        (base32
          "1mbxm957p8gz73kp34vxqhcd2pl716qi5kmlhy5r35aix083ghj8"))))
  (build-system python-build-system)
  (propagated-inputs `(("python-six" ,python-six)))
  (home-page "http://www.pysmt.org")
  (synopsis
    "A solver-agnostic library for SMT Formulae manipulation and solving")
  (description
    "A solver-agnostic library for SMT Formulae manipulation and solving")
  (license #f)))

(define-public python-penaltymodel-maxgap
(package
  (name "python-penaltymodel-maxgap")
  (version "0.4.1")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "penaltymodel-maxgap" version))
      (sha256
        (base32
          "18dfa5k04a2z87mp8krwlcqfqqp1c8kc5ckbh3za96gc0d0piasv"))))
  (build-system python-build-system)
  (native-inputs
   `(("python-penaltymodel" ,python-penaltymodel)))
  (propagated-inputs
   `(("python-dimod" ,python-dimod)
     ("python-dwave-networkx" ,python-dwave-networkx)
     ("python-six" ,python-six)
     ("python-pysmt" ,python-pysmt)))
  (home-page "")
  (synopsis
    "Generates penalty models using smt solvers.")
  (description
    "Generates penalty models using smt solvers.")
  (license license:asl2.0)))

;; FIXME Protobuf, cbc, coinutils, osi, clp, cgl.
; https://github.com/google/or-tools/blob/master/Dependencies.txt
(define-public python-ortools
  (package
    (name "python-ortools")
    (version "6.9.1")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/google/or-tools.git")
              (commit (string-append "v" version))))
        (sha256
          (base32
            "099j1mc7vvry0a2fiz9zvk6divivglzphv48wbw0c6nd5w8hb27c"))))
  (build-system cmake-build-system)
  (inputs
   `(("gflags" ,gflags)
     ("glog" ,glog)))
  (home-page "https://developers.google.com/optimization/")
  (synopsis "Combinatorial optimization")
  (description "Combinatorial optimization.")
  (license license:asl2.0)))

(define-public python-penaltymodel-mip
(package
  (name "python-penaltymodel-mip")
  (version "0.1.3")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "penaltymodel-mip" version))
      (sha256
        (base32
          "0q5qmasbhkv6grg51pnvkwd4gai2wcyg43p2c4d3d397cg63yqdi"))))
  (build-system python-build-system)
  (native-inputs
   `(("python-penaltymodel" ,python-penaltymodel)))
  (propagated-inputs
   `(("python-ortools" ,python-ortools)))
  (home-page
    "https://github.com/dwavesystems/penaltymodel")
  (synopsis
    "Generates penalty models using Google Optimization Tools' Mixed-Integer Programming capability.")
  (description
    "Generates penalty models using Google Optimization Tools' Mixed-Integer Programming capability.")
  (license license:asl2.0)))

(define-public python-dwavebinarycsp
  (package
    (name "python-dwavebinarycsp")
    (version "0.0.6")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwavebinarycsp" version))
        (sha256
          (base32
            "1qydba06b1bmfcyw4bbvxlkvfhbsg5sr4rx879lqq4809zg78wy2"))))
    (build-system python-build-system)
    (propagated-inputs
     `(("python-dimod" ,python-dimod)
       ("python-networkx" ,python-networkx)
       ("python-penaltymodel" ,python-penaltymodel)
       ("python-penaltymodel-cache"
        ,python-penaltymodel-cache)
       ("python-penaltymodel-maxgap"
        ,python-penaltymodel-maxgap)
       ("python-penaltymodel-mip"
        ,python-penaltymodel-mip)
       ("python-six" ,python-six)))
    (home-page "https://github.com/dwavesystems/dwavebinarycsp")
    (synopsis "Solves constraints satisfaction problems with binary quadratic
model samplers")
    (description "This package provides a library to solve constraints
satisfaction problems with binary quadratic model samplers.")
    (license license:asl2.0)))

(define-public python-dwave-ocean-sdk
  (package
    (name "python-dwave-ocean-sdk")
    (version "1.0.1")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "dwave-ocean-sdk" version))
        (sha256
          (base32
            "180qvxjphm8bp3v39j5nplb39i2ckdln9ms4vm4w10inlnkl3pk0"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-dwave-qbsolv" ,python-dwave-qbsolv)))
    (propagated-inputs
     `(("python-dwave-neal" ,python-dwave-neal)
       ("python-dwave-system" ,python-dwave-system)
       ("python-dwavebinarycsp" ,python-dwavebinarycsp)))
    (home-page "https://github.com/dwavesystems/dwave-ocean-sdk")
    (synopsis "Software development kit for open source D-Wave tools")
    (description "This package provides a software development kit for open
source D-Wave tools.")
    (license license:asl2.0)))
