(define-module (wip flatpak)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix build-system gnu)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (gnu packages)
  #:use-module (gnu packages attr)
  #:use-module (gnu packages autotools)
  #:use-module (gnu packages backup)
  #:use-module (gnu packages base)
  #:use-module (gnu packages bash)
  #:use-module (gnu packages bison)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages docbook)
  #:use-module (gnu packages gettext)
  #:use-module (gnu packages glib)
  #:use-module (gnu packages gnupg)
  #:use-module (gnu packages gtk)
  #:use-module (gnu packages gnome)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages nettle)
  #:use-module (gnu packages package-management)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages polkit)
  #:use-module (gnu packages xml)
  #:use-module (gnu packages xorg))

(define-public xxostree
  (package
    (name "xxostree")
    (version "2018.1")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/ostreedev/ostree/archive/"
                            "v" version ".tar.gz"))
        (sha256
          (base32
            "0q86h32arjby3m20i9iif31h808fmkqqmjrf5h0r27d6jfwarsca"))))
    (arguments
     `(#:tests? #f
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'autogen
           (lambda* (#:key inputs #:allow-other-keys)
             (use-modules (system repl debug))
             (set! terminal-width (lambda () 1000000))
             (delete-file-recursively "libglnx")
             (copy-recursively (assoc-ref inputs "libglnx") "libglnx")
             (delete-file-recursively "bsdiff")
             ;(symlink "bsdiff-4.3-endsley" "bsdiff")
             (copy-recursively (assoc-ref inputs "bsdiff") "bsdiff")
             (setenv "NOCONFIGURE" "1")
             (invoke "sh" "autogen.sh"))))))
    (build-system gnu-build-system)
    (inputs
     `(("fuse" ,fuse)
       ("gpgme" ,gpgme)
       ("glib" ,glib)
       ("libext2fs" ,e2fsprogs)
       ("libarchive" ,libarchive)
       ("nettle" ,nettle)
       ; TODO HTTP backend, selinux, libmount
       ("zlib" ,zlib)))
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ("bison" ,bison)
       ("bsdiff" ,(origin
             (method git-fetch)
             (uri (git-reference
                   (url "https://github.com/mendsley/bsdiff.git")
                   (commit "7d70d8f4ff48345bc76e314c9d98da91f78873fa")))
             (sha256
              (base32
               "0ai2kykj8i4bqcy3gry36r3n4ax0fldncfhksl5b293nf2fihnws"))))
       ("gettext-minimal" ,gettext-minimal)
       ("glib" ,glib "bin")
       ("gtk-doc" ,gtk-doc)
       ("libglnx" ,(origin
             (method git-fetch)
             (uri (git-reference
                   (url "https://github.com/GNOME/libglnx.git")
                   (commit "6f1ee5db1400b13a9a0fa0b2274ae34e8710c1aa")))
             (sha256
              (base32
               "137329m4lm3ssmxyyr1cin0xmlpsz4kfk26ydq6i4zf7qz81n64s"))))
       ("libtool" ,libtool)
       ("pkg-config" ,pkg-config)
       ("which" ,which)
       ; ("xsltproc" ,libxslt) ; external urls...
       ))
    (home-page "https://ostree.readthedocs.io/en/latest/manual/introduction/")
    (synopsis "ostree")
    (description "OSTree is an upgrade system for Linux-based operating
systems that performs atomic upgrades of complete filesystem trees.")
    ;; Some are BSD-2.
    ;; Some are LGPL2.0+.
    (license license:gpl2)))

(define-public bubblewrap
  (package
    (name "bubblewrap")
    (version "0.3.0")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/projectatomic/bubblewrap/archive/"
                            "v" version ".tar.gz"))
        (sha256
          (base32
            "0js5jgyfm17z1jj5vsxp9ix9nrq94gn2pzfrz8jva4nbkxginy1i"))))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'autogen
           (lambda _
             (setenv "NOCONFIGURE" "1")
             ;(invoke "libtoolize")
             ;(invoke "aclocal")
             ;(invoke "autoconf" "--add-missing")
             (invoke "automake" "--force-missing" "--add-missing")
             (invoke "sh" "autogen.sh")
             ;(invoke "autoreconf" "-i") ; THE ONLY ONE
             ))
         (replace 'configure
           (lambda* (#:key inputs outputs target #:allow-other-keys)
             (invoke "./configure"
                     (string-append "--prefix=" (assoc-ref outputs "out"))
                     ;(string-append "--host=" target)
             ))))))
    (inputs
     `(("libcap" ,libcap)))
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ("gettext-minimal" ,gettext-minimal)
       ("libtool" ,libtool)))
    (home-page "https://github.com/projectatomic/bubblewrap")
    (synopsis "Unprivileged sandboxing tool")
    (description "Plan 9-style namespace container less featureful than user
namespaces (on purpose).")
    (license license:lgpl2.0+)))

(define-public flatpak
  (package
    (name "flatpak")
    (version "0.10.3")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/flatpak/flatpak/archive/"
                            version ".tar.gz"))
        (sha256
          (base32
            "1aqnnhbzx1bbgk7psxmyqylvis2sciaxcm2y57ylwrmasj9kd4nr"))))
    (build-system gnu-build-system)
    (arguments
     `(#:make-flags
       '("LDFLAGS=-lcap")
       #:configure-flags
       ;; TODO --enable-p2p ?? Then, ostree would require --enable-experimental-api
       (list "--disable-documentation"
             (string-append "--with-xml-catalog="
                            (assoc-ref %build-inputs "docbook-xml")
                            "/xml/dtd/docbook/catalog.xml"))
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'patch-/var/tmp
           (lambda _
             (substitute* '("tests/libtest.sh"
                            "tests/test-run.sh"
                            "tests/testlibrary.c"
                            "buildutil/tap-test"
                            ; ./common/flatpak-dir.c
                            ; ./common/flatpak-run.c
                            )
               (("/var/tmp") "/tmp"))
             (substitute* "tests/libtest.sh"
              (("/usr/bin/") "")
              (("/bin/") ""))
             #t))
         (add-after 'unpack 'autogen
           (lambda* (#:key inputs #:allow-other-keys)
             (use-modules (system repl debug))
             (set! terminal-width (lambda () 1000000))
             (setenv "NOCONFIGURE" "1")
             (delete-file-recursively "libglnx")
             (copy-recursively (assoc-ref inputs "libglnx") "libglnx")
             (invoke "tar" "xf" (assoc-ref inputs "bubblewrap"))
             (delete-file-recursively "bubblewrap")
             (symlink "bubblewrap-0.2.0" "bubblewrap")
             (invoke "sh" "autogen.sh")))
         (add-before 'configure 'fix-docbook
           (lambda* (#:key inputs #:allow-other-keys)
             (substitute* "configure"
               ;; The configure check is overzealous about making sure that
               ;; things are in place -- it uses the xmlcatalog tool to make
               ;; sure that docbook-xsl is available, but this tool can only
               ;; look in one catalog file, unlike the $XML_CATALOG_FILES
               ;; variable that Guix defines.  Fool the test by using the
               ;; docbook-xsl catalog explicitly and get on with life.
               (("\"\\$XML_CATALOG_FILE\" \
\"http://docbook.sourceforge.net/release/xsl/")
                (string-append (car (find-files (assoc-ref inputs "docbook-xsl")
                                                "^catalog.xml$"))
                               " \"http://docbook.sourceforge.net/release/xsl/")))
             #t))
         (add-before 'check 'set-environment-up
           (lambda _
             ;(mkdir-p "/tmp/b")
             (setenv "HOME" "/tmp")
             ;; Note: needs xattr support.
             (setenv "TMPDIR" "/tmp")
             ;;; TODO /dev/fuse
             (mkdir-p "/tmp/ldconfig.d")
             (call-with-output-file "/tmp/ldconfig.d/ldconfig"
               (lambda (port)
                 (display "#!/bin/sh\nexit 0\n" port)))
             (chmod "/tmp/ldconfig.d/ldconfig" #o755)
             (setenv "PATH" (string-append "/tmp/ldconfig.d:" (getenv "PATH")))
             #t)))))
    (inputs
     `(("appstream-glib" ,appstream-glib)
       ("attr" ,attr)
       ("fuse" ,fuse)
       ("glib" ,glib)
       ("gpgme" ,gpgme)
       ("json-glib" ,json-glib)
       ("libarchive" ,libarchive)
       ("libcap" ,libcap)
       ("libseccomp" ,libseccomp)
       ("libsoup" ,libsoup)
       ("libxml2" ,libxml2)
       ("libxau" ,libxau)
       ("nettle" ,nettle)
       ("libostree" ,libostree)
       ("polkit" ,polkit)))
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ("dbus" ,dbus) ; for the tests
       ("docbook-xml" ,docbook-xml-4.1.2)
       ("docbook-xsl" ,docbook-xsl)
       ("fuse" ,fuse) ; for the tests
       ("gettext-minimal" ,gettext-minimal)
       ;("git" ,git)
       ("glib" ,glib "bin")
       ("gnupg" ,gnupg)
       ("gobject-introspection" ,gobject-introspection)
       ("gtk-doc" ,gtk-doc)
       ("libtool" ,libtool)
       ("pkg-config" ,pkg-config)
       ("which" ,which)
       ("libglnx" ,(origin
             (method git-fetch)
             (uri (git-reference
                   (url "https://github.com/GNOME/libglnx.git")
                   (commit "6f1ee5db1400b13a9a0fa0b2274ae34e8710c1aa")))
             (sha256
              (base32
               "137329m4lm3ssmxyyr1cin0xmlpsz4kfk26ydq6i4zf7qz81n64s"))))
       ("bubblewrap" ,(origin
        (method url-fetch)
        (uri (string-append "https://github.com/projectatomic/bubblewrap/archive/"
                            "v0.2.0.tar.gz"))
        (sha256
          (base32
            "17796rqhp5wn3q2lspgqbyk7nk1ixns9m2kgw82j2h2yfyi15mjh"))))
       ("xmltproc" ,libxslt)))
    (home-page "https://www.flatpak.org/")
    (synopsis "Flatpak")
    (description "Flatpak")
    (license license:lgpl2.1+)))
