(define-module (wip freecalypso)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix hg-download)
  #:use-module (guix utils)
  #:use-module (guix build-system python)
  #:use-module (guix build-system gnu)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  ;#:use-module (gnu packages base)
  #:use-module (gnu packages cross-base)
  #:use-module (gnu packages embedded)
  #:use-module (gnu packages gcc)
  #:use-module (gnu packages pkg-config))

(define-public fc-binutils
  (let* ((triplet "arm-elf")
         (base (cross-binutils triplet)))
    (package
      (inherit base)
      (name "fc-binutils")
      (version "2.21.1a")
      (source (origin
               (method url-fetch)
               (uri (string-append "mirror://gnu/binutils/binutils-"
                                   version ".tar.bz2"))
               (sha256
                (base32
                 "0m7nmd7gc9d9md43wbrv65hz6lbi2crqwryzpigv19ray1lzmv6d"))
               (patches
                (search-patches "binutils-2.21.1-elf32-arm-thumb.patch"))))
      (arguments
       (substitute-keyword-arguments (package-arguments base)
        ((#:configure-flags flags)
         `(cons* "--disable-nls" "--disable-werror" ,flags)))))))

(define-public gcc-cross-arm-elf
  (let* ((triplet "arm-elf")
         (base (cross-gcc triplet
                          #:xgcc gcc-4.7 ; FIXME: Use gcc 4.5.4.
                          #:xbinutils fc-binutils)))
    (package
      (inherit base)
      ;FIXME:
      ;(native-inputs
      ; `(,@(package-native-inputs base)
      ;   ("fc-newlib" ,@(package-source fc-newlib))))
      (arguments
       (substitute-keyword-arguments (package-arguments base)
        ((#:configure-flags _)
         `(list (string-append "--target=" ,triplet)
                "--disable-libssp" ; dannym
                "--enable-obsolete"
                "--enable-interwork"
                "--enable-multilib"
                "--with-cpu=arm7tdmi"
                "--with-float=soft"
                "--enable-languages=c"
                ;"--with-newlib"
                "--with-system-zlib"
                "--disable-shared"
                "--disable-nls")))))))

(define (fc-selenite platform build-directory)
  (let ((changeset "3055bcff8050")
        (revision "1"))
    (package
      (name (string-append "fc-selenite-" platform))
      (version (string-append "0.0.1-" revision "." (string-take changeset 7)))
      (source
       (origin
         (method hg-fetch)
         (uri (hg-reference
               (url "https://bitbucket.org/falconian/fc-selenite")
               (changeset changeset)))
         (file-name (string-append name "-" version "-checkout"))
         (sha256
          (base32
           "1mdsh5hbpbk6si1zkq2439k27b0xcwdgc337jqq3lfvlpgpqmkdb"))))
      (build-system gnu-build-system)
      (native-inputs
       `(("fc-gcc" ,gcc-cross-arm-elf)))
      (arguments
       `(#:phases
         (modify-phases %standard-phases
           (add-before 'configure 'build-helpers
             (lambda _
               (chdir "helpers")
               (invoke "make")
               (chdir "..")
               #t))
           (replace 'configure
             (lambda _
               (invoke "./configure-gcc.sh" ,platform)))
           (add-before 'build 'chdir
             (lambda _
               (chdir ,build-directory)
               #t)))))
      (home-page "https://www.freecalypso.org/")
      (synopsis (string-append "FreeCalypso firmware for " platform))
      (description (string-append "This package provides the FreeCalypso "
"firmware for " platform))
      (license (license:non-copyleft "file:///COPYING")))))

(define-public fc-selenite-c11x
  (fc-selenite "c11x" "build-c11x-gcc"))

(define-public fc-selenite-c139
  (fc-selenite "c139" "build-c139-gcc"))

(define-public fc-selenite-c155
  (fc-selenite "c155" "build-c155-gcc"))

(define-public fc-selenite-fcdev3b
  (let ((base (fc-selenite "fcdev3b" "build-fcdev3b-gprs-gcc")))
    (package
      (inherit base)
      (arguments
       (substitute-keyword-arguments (package-arguments base)
        ((#:phases phases)
         `(modify-phases ,phases
            (add-after 'build 'build-ram-loadable-image
              (lambda _
                (invoke "make" "ram"))))))))))

(define-public fc-selenite-gtamodem
  (fc-selenite "gtamodem" "build-gtamodem-gcc"))

(define-public fc-selenite-pirelli
  (fc-selenite "pirelli" "build-pirelli-gcc"))
