(define-module (wip gnudatalanguage)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix build-system cmake)
  #:use-module (guix packages)
  #:use-module (guix utils)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (gnu packages)
  #:use-module (gnu packages algebra)
  #:use-module (gnu packages autotools)
  #:use-module (gnu packages backup)
  #:use-module (gnu packages base)
  #:use-module (gnu packages bash)
  #:use-module (gnu packages bison)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages docbook)
  #:use-module (gnu packages image)
  #:use-module (gnu packages geo)
  #:use-module (gnu packages gettext)
  #:use-module (gnu packages glib)
  #:use-module (gnu packages gtk)
  #:use-module (gnu packages gnome)
  #:use-module (gnu packages imagemagick)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages maths)
  #:use-module (gnu packages ncurses)
  #:use-module (gnu packages readline)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages python)
  #:use-module (gnu packages python-xyz)
  #:use-module (gnu packages wxwidgets))

(define-public plplotx
  (package
    (name "plplotx")
    (version "5.13.0")
    (source (origin
              (method url-fetch)
              (uri (string-append "mirror://sourceforge/plplot/plplot/"
                                  version "%20Source/plplot-" version
                                  ".tar.gz"))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "0asxn9hwyb8wk6mb2gmmb7jn0ma1qdyqzn4giz4x3n83igpbndpc"))))
    (native-inputs
     `(("pkg-config" ,pkg-config)))
    (build-system cmake-build-system)
    (arguments
     `(#:tests? #f ; Comprehensive tests require 4 GB of storage.
       ;; plplot assumes that CMAKE_INSTALL_LIBDIR contains the absolute path.
       #:configure-flags
       (list (string-append "-DCMAKE_INSTALL_LIBDIR="
                            (assoc-ref %outputs "out")
                            "/lib"))))
    (synopsis "Creates scientific plots")
    (description "@code{plplot} provides a library for creating scientific plots.  It supports Unicode characters.")
    (home-page "http://plplot.sourceforge.net/")
    (license license:lgpl2.0+)))

(define-public graphicsmagick-32bppx
  (package
    (inherit graphicsmagick)
    (name "graphicsmagick-32bppx")
    (arguments
     (substitute-keyword-arguments (package-arguments graphicsmagick)
      ((#:configure-flags configure-flags)
       `(list "--with-frozenpaths"
             "--enable-shared=yes"
             "--with-x=yes"
             "--with-quantum-depth=32"
             "--enable-quantum-library-names"
             (string-append "--with-gs-font-dir="
                            (assoc-ref %build-inputs "gs-fonts")
                            "/share/fonts/type1/ghostscript")))))))

(define-public gdlx
  (package
    (name "gdlx")
    (version "0.9.8")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/gnudatalanguage/gdl.git")
                    (commit "ef46c5436b6cd3b571e3162ed8f2a16279d8b2e0")))
              (file-name (string-append name "-" version ".tar.gz"))
              (sha256
               (base32
                "189q7x69aq1jxhq8b26vbxlnrgds18rqa77hjy60dk2q1h6mrb3s"))))
    (build-system cmake-build-system)
    (arguments
     `(#:configure-flags
       '("-DPSLIB=OFF" ; replaced by plplot.
         "-DX11=OFF")
       #:phases
       (modify-phases %standard-phases
         (replace 'check
           (lambda _
             (setenv "CTEST_OUTPUT_ON_FAILURE" "1")
             (invoke "make" "check"))))))
    (inputs
     `(("eigen" ,eigen)
       ("fftw" ,fftw)
       ("fftwf" ,fftwf)
       ("libgeotiff" ,libgeotiff)
       ("gsl" ,gsl)
       ("graphicsmagick-32bpp" ,graphicsmagick-32bpp)
       ("hdf4" ,hdf4-alt)
       ("hdf5" ,hdf5)
       ("libpng" ,libpng)
       ("libtiff" ,libtiff)
       ("ncurses" ,ncurses)
       ("netcdf" ,netcdf)
       ("plplot" ,plplot)
       ("python-2" ,python-2) ; we embed Python.
       ("python2-numpy" ,python2-numpy)
       ("readline" ,readline)
       ("wxwidgets" ,wxwidgets-3.1)
       ("zlib" ,zlib)))
    (synopsis "GNU data language")
    (description "@command{gdl} is a Interactive Data Language (IDL)
compatible compiler.")
    (home-page "https://github.com/gnudatalanguage/gdl")
    (license license:gpl2+)))
