
(define-module (wip gnuradio)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix hg-download)
  #:use-module (guix utils)
  #:use-module (guix build-system cmake)
  #:use-module (guix build-system gnu)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  #:use-module (gnu packages algebra)
  #:use-module (gnu packages base)
  #:use-module (gnu packages boost)
  #:use-module (gnu packages check)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages databases)
  #:use-module (gnu packages documentation)
  #:use-module (gnu packages engineering)
  #:use-module (gnu packages ghostscript)
  #:use-module (gnu packages gstreamer)
  #:use-module (gnu packages gtk)
  #:use-module (gnu packages libffi)
  #:use-module (gnu packages libusb)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages maths)
  #:use-module (gnu packages ncurses)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages perl)
  #:use-module (gnu packages python)
  #:use-module (gnu packages python-xyz)
  #:use-module (gnu packages qt)
  #:use-module (gnu packages shells)
  #:use-module (gnu packages swig)
  #:use-module (gnu packages tcl)
  #:use-module (gnu packages tex)
  #:use-module (gnu packages tls)
  #:use-module (gnu packages wxwidgets)
  #:use-module (gnu packages xml))


;    gnuradio-runtime
;    gnuradio-analog [required by gqrx]
;    gnuradio-digital
;    gnuradio-blocks
;    gnuradio-filter
;    gnuradio-fft
;    gnuradio-audio
;    gnuradio-pmt
;    gr-iqbalance
; http://cgit.osmocom.org/cgit/rtl-sdr/
;gnuradio-osmosdr from http://cgit.osmocom.org/cgit/gr-osmosdr/

(define-public doxygen-1.8.14
  (package (inherit doxygen)
    (name "doxygen-b")
    (version "1.8.14")
    (source (origin
             (method url-fetch)
             (uri (string-append "http://ftp.stack.nl/pub/users/dimitri/"
                                 "doxygen" "-" version ".src.tar.gz"))
             (sha256
              (base32
               "0kcxymbam9jwiyjwyvwdjj0h74lbb6c467szsipzbxjyfl17wxfi"))
             (patches (search-patches "doxygen-test.patch"))))))

(define-public gnuradio
  (package
    (name "gnuradio")
    (version "3.7.12.0")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/gnuradio/gnuradio/archive/v" version ".tar.gz"))
        (sha256
          (base32
            "1sswrdpjfl2ipgvv7k8qb8mph94kigjmbg909d3w6d5r7wk0ifr0"))
        (file-name (string-append name "-" version ".tar.gz"))))
    (outputs '("out" "debug"))
    (build-system cmake-build-system)
    (arguments
     `(#:parallel-tests? #f
       #:configure-flags
       '("-DENABLE_INTERNAL_VOLK=OFF")
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'unbundle-volk
           (lambda _
             (delete-file-recursively "volk")
             ;; Make it dump core when it segfaults in the tests.
             (substitute* "cmake/Modules/GrTest.cmake"
              (("\\bforeach[(]environ [$][{]environs[}][)]") "file(APPEND ${sh_file} \"ulimit -c unlimited\\n\")
foreach(environ ${environs})"))
             #t))
         (add-before 'check 'setenv
           (lambda _
             (setenv "HOME" "/tmp")
             #t))
         (add-before 'install 'delete-irreproducible-files
           (lambda _
             ;; This file contains timestamps.
             (system* "pwd")
             (delete-file "docs/doxygen/html/_formulas.log")
             #t)))))
    (inputs
      `(("alsa-lib" ,alsa-lib)
        ("boost" ,boost)
        ("fftwf" ,fftwf)
        ("gsl" ,gsl)
        ("libusb", libusb)
        ; qt
        ;("wxwidgets" ,wxwidgets) ; -gtk2
        ("python-2", python-2)
        ("python2-numpy" ,python2-numpy) ; for grc
        ("python2-pygtk" ,python2-pygtk) ; for grc
        ("python2-cheetah" ,python2-cheetah) ; for grc
        ("python2-lxml" ,python2-lxml) ; for grc
        ("python2-numpy" ,python2-numpy)
        ("volk" ,volk)
; TODO gui network widgets svg(runtimeonly)/imageformats
))
    (native-inputs
     `(("cppunit" ,cppunit)
       ("doxygen" ,doxygen-1.8.14) ; for docs
       ("gs" ,ghostscript) ; for docs
       ("orc" ,orc)
       ("pkg-config" ,pkg-config)
       ("python-2", python-2)
       ;("python2-cheetah" ,python2-cheetah) ; FIXME maybe remove [since it doesn't show up in the output of grc anyway]
       ("swig" ,swig)
       ("texlive-tiny" ,texlive-tiny))) ; for docs
    (home-page "https://www.gnuradio.org/")
    (synopsis "Software-defined radio")
    (description "@code{gnuradio} contains tools for Software-defined radio.
This includes building blocks for receivers, filters etc.
For this to be useful as radio receiver you also need extra hardware.")
    (license license:gpl3+)))

(define-public gqrx
  (package
    (name "gqrx")
    (version "2.6")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/csete/gqrx/releases/download/v2.6/"
                            "gqrx-sdr-" version "-src.tar.xz"))
        (sha256
          (base32
            "06jq50r0n86448q4l18hhzk5r6s4yzi2i1fc32ybjh06sqydsax8"))))
    (build-system cmake-build-system)
    ; FIXME propagate input qtsvg
    (inputs
     `(("qtbase" ,qtbase)
       ("gnuradio" ,gnuradio)
       ("boost" ,boost)))
    (home-page "http://gqrx.dk/")
    (synopsis "Software-defined radio")
    (description
      "This package contains tools for Software-defined radio.
For this to be useful as radio receiver you also need extra hardware.")
    ;; TODO simplified BSD
    (license license:gpl3+)))
