(define-module (wip pure)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix utils)
  #:use-module (guix build-system gnu)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  #:use-module (gnu packages llvm)
  #:use-module (gnu packages multiprecision))

(define-public pure
  (package
    (name "pure")
    (version "0.67")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/agraef/pure-lang/releases/"
                            "download/pure-" version "/"
                            "pure-" version ".tar.gz"))
        (sha256
          (base32
            "1hlx5dx6n70rbg668yc97yj4jbnrv0dpfw3l9b0ffmcgr76f6ba4"))))
    (build-system gnu-build-system)
    (arguments
     `(#:make-flags (list (string-append "LDFLAGS=-Wl,-rpath="
                                         (assoc-ref %outputs "out")
                                         "/lib"))
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'patch-llvm-lookup
           (lambda _
             (substitute* "configure"
              (("-lLLVM-[$][{]llvm_version[}]")
               "`$LLVMCONF --libs`"))
             #t)))))
    (inputs
     `(("gmp" ,gmp)
       ("llvm" ,llvm-3.5)
       ("mpfr" ,mpfr)))
    (home-page "https://agraef.github.io/pure-lang/")
    (synopsis "Pure programming Language")
    (description "@code{pure} is a programming language based on term
rewriting.  It offers equational definitions with pattern matching,
full symbolic rewriting capabilities, dynamic typing, eager and lazy
evaluation, lexical closures, built-in list and matrix support and
a C interface.")
    (license license:gpl3+)))

;; TODO pure-audio, -avahi, -bonjour, -csv, -doc, -fastcgi, -faust, -ffi, -g2, -gen, -gl, -glpk, -gplot, -gsl, -gtk, -liblo, -lilv, -lv2, -midi, -mpfr, -octave, -odbc, -plugr, -rational, -readline, -reduce, -sockets, -sql3, -stldict, -stllib, -tk-examples, -tk, -xml, purepad, sphinx??, pd-*
