;;; This file is an addendum to GNU Guix.
;;;
;;; GNU Guix is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or (at
;;; your option) any later version.
;;;
;;; GNU Guix is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.

(define-module (wip retdec)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix git-download)
  #:use-module (guix packages)
  #:use-module (gnu packages bison)
  #:use-module (gnu packages check)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages cross-base)
  #:use-module (gnu packages engineering)
  #:use-module (gnu packages flex)
  #:use-module (gnu packages llvm)
  #:use-module (gnu packages perl)
  #:use-module (gnu packages python)
  #:use-module (gnu packages serialization)
  #:use-module (gnu packages tls)
  #:use-module (gnu packages web)
  #:use-module (gnu packages xml)
  #:use-module (guix build-system cmake))

; TODO:
;elfio [avast]
;keystone [keystone-engine on github]
;libdwarf [avast]
;pelib [avast]
;whereami https://github.com/gpakosz/whereami
;yaracpp [avast]
;yaramod [avast] [flex bison]

(define-public retdec
  (package
    (name "retdec")
    (version "3.2")
    (source
      (origin
        (method git-fetch)
        (file-name (string-append name "-" version "-checkout"))
        (uri (git-reference
              (url "https://github.com/avast-tl/retdec.git")
              (commit (string-append "v" version))))
        (sha256
         (base32
          "0chky656lsddn20bnm3pmz6ix20y4a0y8swwr42hrhi01vkhmzrp"))
        (modules '((guix build utils)))
        (snippet
         '(begin
            (delete-file-recursively "deps")
            (mkdir "deps")
            #t))))
    (build-system cmake-build-system)
    (inputs
     `(("bison" ,bison)
       ("capstone" ,capstone)
       ("flex" ,flex)
       ("jsoncpp" ,jsoncpp)
       ("llvm" ,llvm)
       ("openssl" ,openssl)
       ("rapidjson" ,rapidjson)
       ("tinyxml2" ,tinyxml2)
       ("zlib" ,zlib)))
    (native-inputs
     `(("googletest" ,googletest)
       ("perl" ,perl)
       ("python" ,python)))
    (home-page "https://retdec.com/")
    (synopsis "Retargetable Decompiler")
    (description "This package provides a retargetable decompiler based on
LLVM.")
    (license license:expat)))
