(define-module (wip u-boot)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix build-system gnu)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix utils)
  #:use-module (gnu packages)
  #:use-module (gnu packages autotools)
  #:use-module (gnu packages base)
  #:use-module (gnu packages bash)
  #:use-module (gnu packages bootloaders)
  #:use-module (gnu packages cdrom)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages cpio)
  #:use-module (gnu packages curl)
  #:use-module (gnu packages disk)
  #:use-module (gnu packages glib)
  #:use-module (gnu packages gnome)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages mtools)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages tls)
  #:use-module (gnu packages virtualization)
  #:use-module (gnu packages wm))

(define-public rauc
  (package
    (name "rauc")
    (version "1.0")
    (source
     (origin
      (method git-fetch)
      (uri (git-reference
             (url "https://github.com/rauc/rauc.git")
             (commit (string-append "v" version))))
      (file-name (git-file-name name version))
      (sha256
       (base32
        "0wfwmxb661ppvjl0n5f99lhsgygyjqiqdxidzi1483vzw299libd"))))
    (build-system gnu-build-system)
    ;; FIXME: gettext.
    ; FIXME: It uses dbus.
    ; /sbin/mkfs.ext4
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ("glib:bin" ,glib "bin")
       ("libtool" ,libtool)
       ("pkg-config" ,pkg-config)))
    (inputs
     `(("glib" ,glib)
       ("curl" ,curl)
       ("json-glib" ,json-glib)
       ("openssl" ,openssl)))
    (synopsis "Updater for embedded Linux systems")
    (description "RAUC controls the update process on embedded Linux systems.
It consists of a target application that runs as an update client and a
host/target tool that allows you to create, inspect and modify
installation artifacts.")
    (home-page "https://rauc.io/")
    (license license:lgpl2.1)))

(define-public genimagex
  (package
    (name "genimagex")
    (version "10")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/pengutronix/genimage.git")
                    ;(commit "f3bdfaa34c1ca66131ee079c85a68cdac55dbff0")
                    (commit (string-append "v" version))))
              (file-name (string-append name "-" version "-checkout"))
              (sha256
               (base32
                "0had00p2why2l1fl14mq7nbhmmfbd3na4qnnpg36akdy05g67jbn"))
              (patches (search-patches "genimage-find.patch"))))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'guixify
           (lambda* (#:key inputs #:allow-other-keys)
             (substitute* "genimage.c"
              (("cp -a") "cp -r --preserve=ownership,timestamps,links"))
             ;; Note to maintainers: Check ".def =" lines in source.
             (substitute* "config.c"
              (("\\.def = \"cpio\"")
               (string-append ".def = \""
                              (assoc-ref inputs "cpio")
                              "/bin/cpio\""))
              (("\\.def = \"dd\"")
               (string-append ".def = \""
                              (assoc-ref inputs "coreutils")
                              "/bin/dd\""))
              (("\\.def = \"debugfs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "e2fsprogs")
                              "/sbin/debugfs\""))
              (("\\.def = \"e2fsck\"")
               (string-append ".def = \""
                              (assoc-ref inputs "e2fsprogs")
                              "/sbin/e2fsck\""))
              (("\\.def = \"genext2fs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "genext2fs")
                              "/bin/genext2fs\""))
              (("\\.def = \"genisoimage\"")
               (string-append ".def = \""
                              (assoc-ref inputs "cdrkit-libre")
                              "/bin/genisoimage\""))
              (("\\.def = \"mcopy\"")
               (string-append ".def = \""
                              (assoc-ref inputs "mtools")
                              "/bin/mcopy\""))
              (("\\.def = \"mmd\"")
               (string-append ".def = \""
                              (assoc-ref inputs "mtools")
                              "/bin/mmd\""))
              ;;; Note: mkcramfs is obsolete.
              (("\\.def = \"mkdosfs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "dosfstools")
                              "/sbin/mkfs.fat\""))
              (("\\.def = \"mke2fs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "e2fsprogs")
                              "/sbin/mke2fs\""))
              (("\\.def = \"mkfs\\.jffs2\"")
               (string-append ".def = \""
                              (assoc-ref inputs "mtd-utils")
                              "/sbin/mkfs.jffs2\""))
              (("\\.def = \"mkfs\\.ubifs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "mtd-utils")
                              "/sbin/mkfs.ubifs\""))
              (("\\.def = \"mksquashfs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "squashfs-tools")
                              "/bin/mksquashfs\""))
              (("\\.def = \"qemu-img\"")
               (string-append ".def = \""
                              (assoc-ref inputs "qemu")
                              "/bin/qemu-img\""))
              (("\\.def = \"tar\"")
               (string-append ".def = \""
                              (assoc-ref inputs "tar")
                              "/bin/tar\""))
              (("\\.def = \"tune2fs\"")
               (string-append ".def = \""
                              (assoc-ref inputs "e2fsprogs")
                              "/sbin/tune2fs\""))
              (("\\.def = \"ubinize\"")
               (string-append ".def = \""
                              (assoc-ref inputs "mtd-utils")
                              "/sbin/ubinize\""))
              (("\\.def = \"mkimage\"")
               (string-append ".def = \""
                              (assoc-ref inputs "u-boot-tools")
                              "/bin/mkimage\"")))
             (substitute* "test/basic-images.test"
              ;; Work around bug in sharness.sh.
              (("mkdosfs")
               "mkfs.fat")
              ;; Work around bug in sharness.sh.
              (("dd,mkfs\\.fat,mcopy")
               "dd,mkfs_fat,mcopy")
              ;; Should be in the next upstream release.
              (("qemu_img") "qemu-img"))
             (substitute* "util.c"
              (("\"/bin/sh\"")
               (string-append "\"" (assoc-ref inputs "bash") "/bin/sh\"")))
             ;; We don't have /etc/passwd so uid 0 is not known as "root".
             ;; Thus patch it out.
             (substitute* '("test/ext2test.dump"
                            "test/ext3test.dump"
                            "test/ext4test.dump"
                            "test/mke2fs.dump")
              (("root") "unknown"))
             #t))
         (add-before 'check 'setenv-check
           (lambda _
             ;(umask #o0022)
             ;; Our container doesn't provide access to /etc/mtab
             (setenv "EXT2FS_NO_MTAB_OK" "1")
             ;; Make test reproducible
             (setenv "GENIMAGE_MKFJFFS2" "mkfs.jffs2 -U")
             (setenv "GENIMAGE_MKE2FS" "mke2fs -E no_copy_xattrs")
             #t)))))
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ;;; Note: cramfs is obsolete.
       ("dtc" ,dtc) ; for the tests
       ("fakeroot" ,fakeroot) ; for the tests
       ("fdisk" ,fdisk) ; for the tests
       ("pkg-config" ,pkg-config)
       ("util-linux" ,util-linux))) ; for the tests
    (inputs
     `(("bash" ,bash)
       ("cdrkit-libre" ,cdrkit-libre)
       ("cpio" ,cpio)
       ;; Note: invoked by final executable.
       ("coreutils" ,coreutils) ; chmod, dd
       ("dosfstools" ,dosfstools)
       ("e2fsprogs" ,e2fsprogs)
       ("genext2fs" ,genext2fs)
       ("libconfuse" ,libconfuse)
       ("mtd-utils" ,mtd-utils)
       ("mtools" ,mtools)
       ("qemu" ,qemu-minimal)
       ("squashfs-tools" ,squashfs-tools)
       ("tar" ,tar)
       ("u-boot-tools" ,u-boot-tools)))
    (synopsis "Create Flash images according to specification")
    (description "@command{genimage} creates Flash images according to a
specification file.")
    (home-page "https://github.com/pengutronix/genimage")
    (license license:gpl2)))
