(define-module (wip xffm)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix git-download)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (gnu packages admin)
  #:use-module (gnu packages base)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages cross-base)
  #:use-module (gnu packages curl)
  #:use-module (gnu packages embedded)
  #:use-module (guix build-system gnu)
  #:use-module (gnu packages ghostscript)
  #:use-module (gnu packages glib)
  #:use-module (gnu packages gnome)
  #:use-module (gnu packages gtk)
  #:use-module (gnu packages file-systems)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages man)
  #:use-module (gnu packages nfs)
  #:use-module (gnu packages package-management)
  #:use-module (gnu packages perl)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages python)
  #:use-module (gnu packages samba)
  #:use-module (gnu packages xml)
  #:use-module (gnu packages xorg))

(define-public libdbh2
  (package
    (name "libdbh2")
    (version "5.0.22")
    (source (origin
              (method url-fetch)
              (uri (string-append "mirror://sourceforge/dbh/dbh/" version "/"
                                  name "-" version ".tar.gz"))
              (sha256
               (base32
                "1mwidq29k38q5q1cq7mi8jvv332km516n9arbcw1w1qx3iylxxcd"))))
    (build-system gnu-build-system)
    (native-inputs
     `(("gtk-doc" ,gtk-doc)))           ; FIXME make it use it.
    (home-page "http://xffm.org/")
    (synopsis "Disk-based hash tables")
    (description "This package provides a library to create and manage 64-bit
disk based hash tables Associations are made between keys and values so that
for a given a key the value can be found and loaded into memory quickly.")
    (license license:lgpl3+)))

(define-public libtubo0
  (package
    (name "libtubo0")
    (version "5.0.15")
    (source (origin
              (method url-fetch)
              (uri (string-append "mirror://sourceforge/xffm/libtubo/"
                                  name "-" version ".tar.bz2"))
              (sha256
               (base32
                "1nrixkqcv8zp8gc5dgm4ygjz7d7l4d7jafy76ljbv216d97x3xlq"))))
    (build-system gnu-build-system)
    (home-page "http://xffm.org/")
    (synopsis "Inter-process communication via childprocess stdio")
    (description "This package provides a small and simple library to
run any other process in the background and communicate via stdout, stderr
and stdin file descriptors.")
    (license license:gpl3+)))

;; FIXME This refers to rodent a lot - so we need to inline it into rodent.
(define-public librfm5
  (package
    (name "librfm5")
    (version "5.3.16.3")
    (source (origin
              (method url-fetch)
              (uri (string-append "mirror://sourceforge/xffm/" version "/"
                                  name "-" version ".tar.bz2"))
              (sha256
               (base32
                "0lrpxi1350fzbc7lk42ir48maf14qadjk1q9h13558ir9bh6nv9j"))))
    (build-system gnu-build-system)
    (native-inputs
     `(("pkg-config" ,pkg-config)
       ("intltool" ,intltool)
       ("procps" ,procps)))
    (propagated-inputs
     `(("libx11" ,libx11)
       ;;; Propagated because the pkg-config file needs them.
       ("cairo", cairo)
       ("libtubo0" ,libtubo0)
       ("glib" ,glib)
       ("gtk+" ,gtk+)
       ("libdbh2" ,libdbh2)
       ("libxml2" ,libxml2)
       ("libzip" ,libzip)))
    (inputs
     `(("ghostscript" ,ghostscript)
       ("librsvg" ,librsvg)))
    (home-page "http://xffm.org/")
    (synopsis "File manager library")
    (description "File manager library.")
    (license license:gpl3+)))

(define-public rodent
  (package
    (name "rodent")
    (version "5.3.16.3")
    (source (origin
              (method url-fetch)
              (uri (string-append "mirror://sourceforge/xffm/" version "/"
                                  name "-" version ".tar.bz2"))
              (sha256
               (base32
                "1mpkfli4l1q469x2a2zhh5fj71ir3rgs7l6sjhdayq1vxyjglvs5"))))
    (build-system gnu-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'fix-translations
           (lambda _
             ;; The translations need those - but they don't exist.
             (invoke "touch" "apps/rodent-fuse/src/obex-dialog.i")
             (invoke "touch" "apps/rodent-fuse/src/obex-submodule.c")))
         (add-after 'unpack 'patch-paths
           (lambda* (#:key inputs outputs #:allow-other-keys)
             (let ((patch-argv0
                    (lambda (inputs)
                      (lambda* (value package #:optional (bindir "bin"))
                        (substitute* (find-files "." "\\.[ci]$")
                         (((string-append "g_find_program_in_path[ ]*[(]\"" value "\"[)]"))
                          (string-append "g_strdup (\"" (assoc-ref inputs package) "/"
                                         bindir "/" value "\")"))
                         (((string-append "\"" value "\"")
                          (string-append "\"" (assoc-ref inputs package) "/"
                                         bindir "/" value "\""))))))))
               (with-fluids ((%default-port-encoding #f))
                 (substitute* '("a_configure"
                                "apps/rodent-diff/configure"
                                "apps/rodent-fm/configure"
                                "apps/fgr/configure" ; ISO8859-1
                                "apps/rodent-samba/configure"
                                "apps/rodent-fuse/configure"
                                "apps/rodent-pkg/configure"
                                "apps/rodent-dotdesktop/configure"
                                "apps/rodent-fgr/configure"
                                "apps/rodent-ps/configure"
                                "apps/rodent-fstab/configure"
                                "apps/rodent-iconmgr/configure")
                   (("/usr/bin/file") "file")
                   (("/usr/bin/objformat") "objformat")
                   (("/usr/bin/uname") "uname")))
               ;; TODO Maybe patch out "bcrypt", "pkg", "rpm", "yum", "zypper".
               (let ((patch-argv0 (patch-argv0 inputs)))
                 (patch-argv0 "fusermount" "fuse")
                 (patch-argv0 "umount" "util-linux")
                 (patch-argv0 "rmdir" "util-linux")
                 ;; FIXME Maybe breaks the smb_check_program call ??
                 (patch-argv0 "smbclient" "samba")
                 (patch-argv0 "nmblookup" "samba")
                 (patch-argv0 "rpm" "rpm")
                 (patch-argv0 "grep" "grep")
                 (patch-argv0 "diff" "diffutils")
                 (patch-argv0 "sshfs" "sshfs-fuse")
                 (patch-argv0 "sudo" "sudo")
                 (patch-argv0 "mount" "util-linux")
                 ;; XXX Some modules use "mount -t ecryptfs".
                 (patch-argv0 "mount.ecryptfs" "ecryptfs-utils" "sbin")
                 (patch-argv0 "mount.nfs" "nfs-utils" "sbin")
                 (patch-argv0 "mount.cifs" "cifs-utils" "sbin")
                 (patch-argv0 "curlftpfs" "curlftpfs")
                 (patch-argv0 "cp" "coreutils")
                 (patch-argv0 "mv" "coreutils")
                 (patch-argv0 "ln" "coreutils")
                 (patch-argv0 "rm" "coreutils")
                 (patch-argv0 "chown" "coreutils")
                 (patch-argv0 "chmod" "coreutils")
                 (patch-argv0 "shred" "coreutils")
                 (patch-argv0 "touch" "coreutils")
                 (patch-argv0 "kill" "coreutils")
                 (patch-argv0 "ps" "procps")
                 ; FIXME (patch-argv0 '("rodent-fuse/src/ftp-submodule.c") "httpproxy" "????")
                 (patch-argv0 "renice" "util-linux"))
               (let ((patch-argv0 (patch-argv0 outputs)))
                 (patch-argv0 "rodent" "out")
                 (patch-argv0 "rodent-desk" "out")
                 (patch-argv0 "rodent-plug" "out")
                 (patch-argv0 "rodent-fm" "out")
                 (patch-argv0 "rodent-anim" "out")
                 (patch-argv0 "fgr" "out"))
               #t)))
         (replace 'configure
           (lambda* (#:key inputs outputs #:allow-other-keys)
             (setenv "CONFIG_SHELL" (which "sh"))
             ;; "configure" is a Perl script.
             (invoke "./configure"
                     (string-append "--prefix="
                                    (assoc-ref outputs "out"))))))))
    (inputs
     `(("cifs-utils" ,cifs-utils)
       ("coreutils" ,coreutils)
       ("curlftpfs" ,curlftpfs)
       ("diffutils" ,diffutils)
       ("ecryptfs-utils" ,ecryptfs-utils)
       ("fuse" ,fuse)
       ("grep" ,grep)
       ("librfm5" ,librfm5)
       ("man-db" ,man-db)
       ("nfs-utils" ,nfs-utils)
       ("procps" ,procps)
       ("python-2" ,python-2)
       ("rpm" ,rpm)
       ("sshfs-fuse" ,sshfs-fuse)
       ("sudo" ,sudo)
       ("util-linux" ,util-linux)
       ("samba" ,samba)))
    (native-inputs
     `(("ghostscript" ,ghostscript)
       ("intltool" ,intltool)
       ("perl" ,perl)
       ("pkg-config" ,pkg-config)
       ("procps" ,procps)))
    (home-page "http://xffm.org/")
    (synopsis "File manager")
    (description "File manager.")
    (license license:gpl3+)))
